@@ -25,13 +25,16 @@
 #include <gtk/gtk.h>
 #include <NvCtrlAttributes.h>
 
+#include "dfp_banner.h"
 #include "ctkimage.h"
 
 #include "ctkdisplaydevice-dfp.h"
 
 #include "ctkimagesliders.h"
+#include "ctkedid.h"
 #include "ctkconfig.h"
 #include "ctkhelp.h"
+#include "ctkutils.h"
 
 
 static GtkWidget *make_scaling_radio_button(CtkDisplayDeviceDfp
@@ -71,6 +74,7 @@ static void dfp_update_received(GtkObject *object, gpointer arg1,
 
 #define __SCALING (1<<0)
 #define __DITHERING (1<<1)
+#define __INFO (1<<2)
 
 
 static const char *__scaling_help =
@@ -85,6 +89,10 @@ static const char *__dithering_help =
 "properly display on a flatpanel; this option allows "
 "you to control the dithering behavior.";
 
+static const char *__info_help = 
+"This section describes basic informations about the "
+"DVI connection to the digital flat panel.";
+
 
 GType ctk_display_device_dfp_get_type(void)
 {
@@ -127,7 +135,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     CtkDisplayDeviceDfp *ctk_display_device_dfp;
     GtkWidget *banner;
     GtkWidget *frame;
-    GtkWidget *hbox, *vbox;
+    GtkWidget *hbox, *vbox, *tmpbox;
     GtkWidget *label;
     GtkWidget *eventbox;
 
@@ -136,6 +144,9 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
     GtkWidget *radio2;
     GtkWidget *radio3;
     GtkWidget *alignment;
+    GtkWidget *edid;
+    
+    GtkWidget *table;
     
     ReturnStatus ret;
     
@@ -153,7 +164,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
 
     /* banner */
 
-    banner = ctk_banner_image_new(BANNER_ARTWORK_DFP);
+    banner = ctk_banner_image_new(&dfp_banner_image);
     gtk_box_pack_start(GTK_BOX(object), banner, FALSE, FALSE, 0);
     
     /*
@@ -183,11 +194,74 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                            "The Reset Hardware Defaults button restores "
                            "the DFP settings to their default values.");
 
-    /* create the hbox to store scaling and dithering */
+    /* create the hbox to store dfp info, scaling and dithering */
 
     hbox = gtk_hbox_new(FALSE, FRAME_PADDING);
-    gtk_box_pack_start(GTK_BOX(object), hbox, TRUE, TRUE, FRAME_PADDING);
+    gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, FRAME_PADDING);
 
+    /* DFP info */
+    ret = NvCtrlGetDisplayAttribute(handle, display_device_mask,
+                                    NV_CTRL_FLATPANEL_CHIP_LOCATION, &val);
+    if (ret == NvCtrlSuccess) {
+        char *chip_location, *link, *signal;
+        gint tmp;
+        
+        /* NV_CTRL_FLATPANEL_CHIP_LOCATION */
+        ret = NvCtrlGetDisplayAttribute(handle, display_device_mask, 
+                                        NV_CTRL_FLATPANEL_CHIP_LOCATION, &tmp);
+        chip_location = NULL;
+        if (ret == NvCtrlSuccess) {
+            if (tmp == NV_CTRL_FLATPANEL_CHIP_LOCATION_INTERNAL) chip_location = "Internal";
+            if (tmp == NV_CTRL_FLATPANEL_CHIP_LOCATION_EXTERNAL) chip_location = "External";
+        }
+         
+        ret = NvCtrlGetDisplayAttribute(handle, display_device_mask, 
+                                        NV_CTRL_FLATPANEL_LINK, &tmp);
+        link = NULL;
+        if (ret == NvCtrlSuccess) {
+            if (tmp == NV_CTRL_FLATPANEL_LINK_SINGLE) link = "Single";
+            if (tmp == NV_CTRL_FLATPANEL_LINK_DUAL) link = "Dual";
+        }
+        
+        ret = NvCtrlGetDisplayAttribute(handle, display_device_mask, 
+                                        NV_CTRL_FLATPANEL_SIGNAL, &tmp);
+        signal = NULL;
+        if (ret == NvCtrlSuccess) {
+            if (tmp == NV_CTRL_FLATPANEL_SIGNAL_LVDS) signal = "LVDS";
+            if (tmp == NV_CTRL_FLATPANEL_SIGNAL_TMDS) signal = "TMDS";
+        }
+          
+        frame = gtk_frame_new("Flat Panel Information");
+        gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
+        
+        table = gtk_table_new(3, 2, FALSE);
+
+        /*
+         * insert a vbox between the frame and the table, so that the
+         * table doesn't expand to fill all of the space within the
+         * frame
+         */
+
+        tmpbox = gtk_vbox_new(FALSE, 0);
+        gtk_box_pack_start(GTK_BOX(tmpbox), table, FALSE, FALSE, 0);
+
+        gtk_container_add(GTK_CONTAINER(frame), tmpbox);
+        
+        gtk_table_set_row_spacings(GTK_TABLE(table), 3);
+        gtk_table_set_col_spacings(GTK_TABLE(table), 15);
+
+        gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+
+    
+        add_table_row(table, 0, 0, "Chip location:", chip_location);
+        add_table_row(table, 1, 0, "DVI connection link:", link);
+        add_table_row(table, 2, 0, "Signal:", signal);
+        
+        ctk_display_device_dfp->active_attributes |= __INFO;
+    }
+    else
+        ctk_display_device_dfp->active_attributes &= ~__INFO;
+    
     /* FlatPanel Scaling */
     
     ret = NvCtrlGetDisplayAttribute(handle, display_device_mask,
@@ -197,7 +271,7 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
         frame = gtk_frame_new("FlatPanel Scaling");
         eventbox = gtk_event_box_new();
         gtk_container_add(GTK_CONTAINER(eventbox), frame);
-        gtk_box_pack_start(GTK_BOX(hbox), eventbox, TRUE, TRUE, 0);
+        gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
     
         ctk_config_set_tooltip(ctk_config, eventbox, __scaling_help);
 
@@ -309,6 +383,18 @@ GtkWidget* ctk_display_device_dfp_new(NvCtrlAttributeHandle *handle,
                            FALSE, FALSE, 0);
     }
 
+    /* pack the EDID button */
+
+    edid = ctk_edid_new(handle, ctk_config, ctk_event,
+                        ctk_display_device_dfp->reset_button,
+                        display_device_mask, name);
+    if (edid) {
+        gtk_box_pack_start(GTK_BOX(object), edid, FALSE, FALSE, 0);
+        ctk_display_device_dfp->edid_available = TRUE;
+    } else {
+        ctk_display_device_dfp->edid_available = FALSE;
+    }
+    
     /* show the page */
 
     gtk_widget_show_all(GTK_WIDGET(object));
@@ -731,6 +817,28 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
     
     ctk_help_title(b, &i, "%s Help", ctk_display_device_dfp->name);
     
+    if (ctk_display_device_dfp->active_attributes & __INFO) {
+        ctk_help_heading(b, &i, "FlatPanel Information");
+        ctk_help_para(b, &i, __info_help);
+        
+        ctk_help_term(b, &i, "Chip Location");
+        ctk_help_para(b, &i, "Report whether the flatpanel is driven by "
+                      "the on-chip controller (internal), or a "
+                      " separate controller chip elsewhere on the "
+                      "graphics board (external)");
+                      
+        ctk_help_term(b, &i, "Link");
+        ctk_help_para(b, &i, "Report whether the specified display device "
+                      "is driven by a single link or dual link DVI "
+                      "connection.");
+        
+        ctk_help_term(b, &i, "Signal");
+        ctk_help_para(b, &i, "Report whether the flatpanel is driven by "
+                       "an LVDS or TMDS signal");
+        
+        ret = TRUE;
+    }
+    
     if (ctk_display_device_dfp->active_attributes & __SCALING) {
         ctk_help_heading(b, &i, "FlatPanel Scaling");
         ctk_help_para(b, &i, __scaling_help);
@@ -770,10 +878,14 @@ GtkTextBuffer *ctk_display_device_dfp_create_help(GtkTextTagTable *table,
     }
 
     if (ctk_display_device_dfp->image_sliders) {
-        ret |= add_image_sharpening_help
+        ret |= add_image_sliders_help
             (CTK_IMAGE_SLIDERS(ctk_display_device_dfp->image_sliders), b, &i);
     }
     
+    if (ctk_display_device_dfp->edid_available) {
+        add_acquire_edid_help(b, &i);
+    }
+
     if (!ret) {
         ctk_help_para(b, &i, "There are no configurable options available "
                       "for %s.", ctk_display_device_dfp->name);
